@@ -5,15 +5,12 @@ home.dash.web
 The web controllers for rendering login, logout and dashboard screens.
 """
 
-from collections import OrderedDict
-
 from flask import render_template, Blueprint, request, redirect, url_for, flash
 from flask.ext.login import login_required, login_user, logout_user
-from sqlalchemy import sql
 
-from home import db
+from home import redis_series
 from home.dash.models import User
-from home.ts.models import Area, Device, Series
+from home.ts.models import Area, DeviceSeries, Device
 from home.webcam.models import Webcam
 
 
@@ -35,49 +32,29 @@ def inject_webcams():
 @login_required
 def dashboard(device_name=None):
 
-    series_map = {s.id: s for s in Series.query.all()}
-
-    query = """ SELECT
-    device.id as device_id, area.name as device_name,
-    series.id as series_id, series.name as series_name,
-    ARRAY (
-        SELECT hstore(data_point)
-        FROM data_point
-        WHERE data_point.device_series_id = device_series.id
-        ORDER BY data_point.created_at DESC
-        LIMIT 30
-    ) as latest
-    FROM device_series
-    JOIN device ON device.id = device_series.device_id
-    JOIN series ON series.id = device_series.series_id
-    JOIN area ON area.id = device.area_id"""
+    device_series = DeviceSeries.query
 
     if device_name:
-        kwargs = {'device_name': device_name}
-        query += "\n    WHERE area.name = :device_name"
-    else:
-        kwargs = {}
+        device_series = device_series\
+            .filter(Area.name == device_name)\
+            .join(DeviceSeries.device)\
+            .join(Device.area)\
+            .order_by(Area.name)
 
-    query += "\n    ORDER BY series_name, device_name"
+    latest, updated = redis_series.status()
 
-    device_series = db.engine.execute(sql.text(query), **kwargs)
+    device_series_values = []
 
-    structured = OrderedDict()
+    for ds in device_series:
 
-    for row in device_series:
+        key = "D-{0}:S-{1}".format(ds.device_id, ds.series_id)
 
-        structured[row.device_name] = structured.get(row.device_name, [])
+        latest_value = latest.get(key)
+        updated_at = updated.get(key)
+        latest_values = ",".join(redis_series.latest(key))
 
-        structured[row.device_name].append({
-            'series_id': row.series_id,
-            'model': series_map[row.series_id],
-            'device_id': row.device_id,
-            'name': row.series_name,
-            'value': row.latest[0]['value'],
-            'created_at': row.latest[0]['created_at'],
-            'data_points': row.latest,
-            'sparkline': ','.join(reversed([d['value'] for d in row.latest]))
-        })
+        line = (ds, latest_value, updated_at, latest_values)
+        device_series_values.append(line)
 
     if device_name is not None:
         template = 'device.html'
@@ -86,7 +63,7 @@ def dashboard(device_name=None):
 
     return render_template(
         template,
-        device_readings=structured,
+        device_readings=device_series_values,
         device_name=device_name,
     )
 
